{ Copyright (C) 09/09/2004 00:00, Luis R. Hilario B. <info@luisdigital.com>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}

unit txtdb;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, util;

Type
  TTXTDB = Class(TObject)
  private
  FFieldSizeTotal,
  FRecordIndex,
  FRecordCount: Integer;
  FArchivo,
  FEncabezado: String;
  FStringList,
  FInfo,
  FCampo,
  FTipo,
  FSize: TStrings;
  function GetRecordData(const Field: String): String;
  function GetRecordDataR(const Field: String): Real;
  procedure SetRecordDataR(const Field: String; const AValue: Real);
  function GetRecordDataI(const Field: String): Integer;
  procedure SetRecordDataI(const Field: String; const AValue: Integer);
  procedure SetRecordData(const Field: String; const AValue: String);
  protected
  property Datos: TStrings Read FStringList Write FStringList;
  public
  constructor Create(Const ArchivoNombre: String);
  function ReadTXTDB: Integer;
  procedure WriteTXTDB;
  property Encabezado: String Read FEncabezado;
  property Campo: TStrings Read FCampo;
  property RecordIndex: Integer Read FRecordIndex Write FRecordIndex;
  property RecordCount: Integer Read FRecordCount;
  property RecordData[const Field: String] : String Read GetRecordData
                                                    Write SetRecordData;
  property RecordDataR[const Field: String] : Real Read GetRecordDataR
                                                   Write SetRecordDataR;
  property RecordDataI[const Field: String] : Integer Read GetRecordDataI
                                                    Write SetRecordDataI;
  property Archivo: String Read FArchivo Write FArchivo;
  function FieldSize(const Field: String): Integer;
  function FieldType(const Field: String): Char;
//  function FieldName(const Index: Integer): String;
  procedure RecordAdd;
  function RecordDelete: Boolean;
  function DataExist(const Field, Data: String): Boolean;
  destructor Destroy; override;
end;

implementation

constructor TTXTDB.Create(Const ArchivoNombre: String);
begin
  FArchivo := ArchivoNombre;
  FStringList := TStringList.Create;
end;

function TTXTDB.FieldSize(const Field: String): Integer;
var
  I, Code: Integer;

begin
  Result := FCampo.IndexOf(Field);
  if Result < 0 then exit;
  Val(FSize[Result], I, Code);
  if Code <> 0 then exit(-2);
  Result := I;
end;

function TTXTDB.FieldType(const Field: String): Char;
var
  Index: Integer;

begin
  Index := FCampo.IndexOf(Field);
  if Index < 0 then Result := 'X' else Result := FTipo[Index][1];
end;

procedure TTXTDB.RecordAdd;
var
  NewRecord: String;

begin
  SetLength(NewRecord, FFieldSizeTotal);
  FillChar(NewRecord[1], FFieldSizeTotal, ' ');
  FRecordIndex := Datos.Add(NewRecord);
  FRecordCount := Datos.Count -1;
end;

function TTXTDB.GetRecordData(const Field: String): String;
var
  y, Pos, Index: Integer;

begin
  if FRecordIndex < 1 then exit('I');
  y := 0;
  Pos := 1;
  Index := FCampo.IndexOf(Field);
  if Index < 0 then exit('X');
  while y <> Index do
  begin
    Pos := Pos + FieldSize(FCampo[y]);
    inc(y);
  end;
  Result := Copy(Datos[FRecordIndex], Pos, FieldSize(FCampo[Index]));
  Result := Trim(Result);
end;

function TTXTDB.GetRecordDataR(const Field: String): Real;
var
  R: Real;
  Code: Word;

begin
  Val(GetRecordData(Field), R, Code);
  if Code <> 0 then Result := 0 else Result := R;
end;

procedure TTXTDB.SetRecordDataR(const Field: String; const AValue: Real);
begin
  SetRecordData(Field, NumStr(AValue));
end;

function TTXTDB.GetRecordDataI(const Field: String): Integer;
var
  I: Integer;
  Code: Word;
begin
  Val(GetRecordData(Field), I, Code);
  if Code <> 0 then Result := 0 else Result := I;
end;

procedure TTXTDB.SetRecordDataI(const Field: String; const AValue: Integer);
var s: string;

begin
  Str(AValue, s);
  SetRecordData(Field, s);
end;

procedure TTXTDB.SetRecordData(const Field: String; const AValue: String);
var
  y, Pos, Index: Integer;
  Value, Temp: String;

begin
  if FRecordIndex < 1 then exit;
  y := 0;
  Pos := 1;
  Index := FCampo.IndexOf(Field);
  if Index < 0 then exit;
  while y <> Index do
  begin
    Pos := Pos + FieldSize(FCampo[y]);
    inc(y);
  end;
  Temp := Datos[FRecordIndex];
  Delete(Temp, Pos, FieldSize(FCampo[Index]));
  Value := AValue;
  While Length(Value) < FieldSize(FCampo[Index]) do Value := Value + ' ';
  SetLength(Value, FieldSize(FCampo[Index]));
  Insert(Value, Temp, Pos);
  Datos[FRecordIndex] := Temp;
end;

function TTXTDB.RecordDelete: Boolean;
begin
  if FRecordIndex < 1 then exit(False);
  Datos.Delete(FRecordIndex);
  FRecordCount := Datos.Count -1;
  WriteTXTDB;
  Result := True;
end;

function TTXTDB.DataExist(const Field, Data: String): Boolean;
var
  Index, TempIndex: Integer;
  TempData: String;

begin
  if FRecordIndex < 1 then exit(False);
  Index := 0;
  Result := False;
  TempIndex := FRecordIndex;
  TempData := Trim(Data);
  Repeat
  inc(Index);
  FRecordIndex := Index;
  Until (GetRecordData(Field) = TempData) Or (FRecordCount = Index);
  if GetRecordData(Field) = TempData then Result := True
  else FRecordIndex := TempIndex;
end;

function TTXTDB.ReadTXTDB: Integer;
var
  x, y,
  size, I, Code: Integer;
  Temp: String;

begin
  FStringList.LoadFromFile(FArchivo);
  Result := 0;
  x := 1;
  FFieldSizeTotal := 0;
  FRecordCount := Datos.Count -1;
  if FRecordCount = 0 then FRecordIndex := 0 else FRecordIndex := 1;
  size := Length(Datos[0]);
  FInfo := TStringList.Create;
  Repeat
  Temp := '';
    Repeat
      Temp := Temp + Datos[0][x];
      inc(x);
    Until (Datos[0][x] = ' ') Or (x > size);
  FInfo.Add(Temp);
  if Datos[0][x] = ' ' then inc(x);
  Until x > size;

  FEncabezado := FInfo[0];

  FCampo := TStringList.Create;
  FTipo := TStringList.Create;
  FSize := TStringList.Create;

  for y := 1 to FInfo.Count -1 do
  begin
      x := 1;
      Temp := '';
      Repeat
        Temp := Temp + FInfo[y][x];
        inc(x);
      Until FInfo[y][x] = '-';
      FCampo.Add(Temp); inc(x);
      FTipo.Add(FInfo[y][x]); inc(x);
      Temp := '';
      Size := Length(FInfo[y]);
      Repeat
        Temp := Temp + FInfo[y][x];
        inc(x);
      Until x > Size;
      FSize.Add(Temp);
      Val(Temp, I, Code);
      if Code <> 0 then exit(-2);
      FFieldSizeTotal := FFieldSizeTotal + I;
  end;
end;

procedure TTXTDB.WriteTXTDB;
begin
  Datos.SaveToFile(Archivo);
end;

destructor TTXTDB.Destroy;
begin
  FInfo.Free;
  FCampo.Free;
  FTipo.Free;
  FSize.Free;
  FStringList.Free;
  inherited Destroy;
end;

end.

